// Copyright 2022 hjiayz
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//     http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
use criterion::{black_box, criterion_group, criterion_main, Criterion};
use regexp2rust_macro::regex2rust;
use regex::bytes::Regex;

const DNA : &'static str = include_str!("regexdna.txt");

const IDENTNAME : &'static str = "regexdna";

regex2rust!( find_new_lines =  ">[^\n]*\n|\n" "ug" );
regex2rust!( variant1 = "agggtaaa|tttaccct" "ug" );
regex2rust!( variant2 = "[cgt]gggtaaa|tttaccc[acg]" "ug" );
regex2rust!( variant3 = "a[act]ggtaaa|tttacc[agt]t" "ug" );
regex2rust!( variant4 = "ag[act]gtaaa|tttac[agt]ct" "ug" );
regex2rust!( variant5 = "agg[act]taaa|ttta[agt]cct" "ug" );
regex2rust!( variant6 = "aggg[acg]aaa|ttt[cgt]ccct" "ug" );
regex2rust!( variant7 = "agggt[cgt]aa|tt[acg]accct" "ug" );
regex2rust!( variant8 = "agggta[cgt]a|t[acg]taccct" "ug" );
regex2rust!( variant9 = "agggtaa[cgt]|[acg]ttaccct" "ug" );
regex2rust!( subst1 = "B" "ug" );
regex2rust!( subst2 = "D" "ug" );
regex2rust!( subst3 = "H" "ug" );
regex2rust!( subst4 = "K" "ug" );
regex2rust!( subst5 = "M" "ug" );
regex2rust!( subst6 = "N" "ug" );
regex2rust!( subst7 = "R" "ug" );
regex2rust!( subst8 = "S" "ug" );
regex2rust!( subst9 = "V" "ug" );
regex2rust!( subst10 = "W" "ug" );
regex2rust!( subst11 = "Y" "ug" );

fn regex2rustbench(s:&str) {/*
    assert_eq!(find_new_lines::Iter::new(s,0).count(),83337);
    assert_eq!(variant1::Iter::new(s,0).count(),32);
    assert_eq!(variant2::Iter::new(s,0).count(),115);
    assert_eq!(variant3::Iter::new(s,0).count(),368);
    assert_eq!(variant4::Iter::new(s,0).count(),254);
    assert_eq!(variant5::Iter::new(s,0).count(),466);
    assert_eq!(variant6::Iter::new(s,0).count(),135);
    assert_eq!(variant7::Iter::new(s,0).count(),137);
    assert_eq!(variant8::Iter::new(s,0).count(),139);
    assert_eq!(variant9::Iter::new(s,0).count(),197);
    assert_eq!(subst1::Iter::new(s,0).count(),29963);
    assert_eq!(subst2::Iter::new(s,0).count(),29987);
    assert_eq!(subst3::Iter::new(s,0).count(),30004);
    assert_eq!(subst4::Iter::new(s,0).count(),29974);
    assert_eq!(subst5::Iter::new(s,0).count(),29979);
    assert_eq!(subst6::Iter::new(s,0).count(),29959);
    assert_eq!(subst7::Iter::new(s,0).count(),30012);
    assert_eq!(subst8::Iter::new(s,0).count(),29994);
    assert_eq!(subst9::Iter::new(s,0).count(),29996);
    assert_eq!(subst10::Iter::new(s,0).count(),29984);*/
    assert_eq!(subst11::Iter::new(s,0).count(),29947);
}

fn regexbench(s:&str,regexs:&[Regex]){
    let s = s.as_bytes();
    /*
    assert_eq!(regexs[0].find_iter(s).count(),83337);
    assert_eq!(regexs[1].find_iter(s).count(),32);
    assert_eq!(regexs[2].find_iter(s).count(),115);
    assert_eq!(regexs[3].find_iter(s).count(),368);
    assert_eq!(regexs[4].find_iter(s).count(),254);
    assert_eq!(regexs[5].find_iter(s).count(),466);
    assert_eq!(regexs[6].find_iter(s).count(),135);
    assert_eq!(regexs[7].find_iter(s).count(),137);
    assert_eq!(regexs[8].find_iter(s).count(),139);
    assert_eq!(regexs[9].find_iter(s).count(),197);
    assert_eq!(regexs[10].find_iter(s).count(),29963);
    assert_eq!(regexs[11].find_iter(s).count(),29987);
    assert_eq!(regexs[12].find_iter(s).count(),30004);
    assert_eq!(regexs[13].find_iter(s).count(),29974);
    assert_eq!(regexs[14].find_iter(s).count(),29979);
    assert_eq!(regexs[15].find_iter(s).count(),29959);
    assert_eq!(regexs[16].find_iter(s).count(),30012);
    assert_eq!(regexs[17].find_iter(s).count(),29994);
    assert_eq!(regexs[18].find_iter(s).count(),29996);
    assert_eq!(regexs[19].find_iter(s).count(),29984);*/
    assert_eq!(regexs[20].find_iter(s).count(),29947);
}

fn criterion_benchmark(c: &mut Criterion) {
    let regexs = [
        ">[^\n]*\n|\n",
        "agggtaaa|tttaccct",
        "[cgt]gggtaaa|tttaccc[acg]",
        "a[act]ggtaaa|tttacc[agt]t",
        "ag[act]gtaaa|tttac[agt]ct",
        "agg[act]taaa|ttta[agt]cct",
        "aggg[acg]aaa|ttt[cgt]ccct",
        "agggt[cgt]aa|tt[acg]accct",
        "agggta[cgt]a|t[acg]taccct",
        "agggtaa[cgt]|[acg]ttaccct",
        "B",
        "D",
        "H",
        "K",
        "M",
        "N",
        "R",
        "S",
        "V",
        "W",
        "Y",
    ].map(|reg|Regex::new(reg).unwrap());
    c.bench_function("regex", |b| b.iter(|| regexbench(black_box(DNA),&regexs)));
    c.bench_function("regex2rust", |b| b.iter(|| regex2rustbench(black_box(DNA))));


}

criterion_group!(benches, criterion_benchmark);
criterion_main!(benches);